@@ -17,20 +17,22 @@ modify it under the same terms as Perl itself.
 //------------------------------------------------------------------------------
 
 if (typeof Jemplate == 'undefined') {
-    Jemplate = function() {
+    var Jemplate = function() {
         this.init.apply(this, arguments);
     };
 }
 
-if (! Jemplate.templateMap)
-    Jemplate.templateMap = {};
-
 Jemplate.process = function() {
     var jemplate = new Jemplate();
     return jemplate.process.apply(jemplate, arguments);
 }
 
-proto = Jemplate.prototype;
+;(function(){
+
+if (! Jemplate.templateMap)
+    Jemplate.templateMap = {};
+
+var proto = Jemplate.prototype = {};
 
 proto.init = function(config) {
     this.config = config ||
@@ -68,6 +70,9 @@ proto.process = function(template, data, output) {
     context.__filter__ = new Jemplate.Filter();
     context.__filter__.config = this.config;
 
+    context.__plugin__ = new Jemplate.Plugin();
+    context.__plugin__.config = this.config;
+
     var result;
 
     var proc = function(input) {
@@ -84,7 +89,7 @@ proto.process = function(template, data, output) {
             return result;
         if (typeof output == 'function') {
             output(result);
-            return;
+            return null;
         }
         if (typeof(output) == 'string' || output instanceof String) {
             if (output.match(/^#[\w\-]+$/)) {
@@ -93,12 +98,12 @@ proto.process = function(template, data, output) {
                 if (typeof element == 'undefined')
                     throw('No element found with id="' + id + '"');
                 element.innerHTML = result;
-                return;
+                return null;
             }
         }
         else {
             output.innerHTML = result;
-            return;
+            return null;
         }
 
         throw("Invalid arguments in call to Jemplate.process");
@@ -109,8 +114,10 @@ proto.process = function(template, data, output) {
     if (typeof data == 'function')
         data = data();
     else if (typeof data == 'string') {
-        Ajax.get(data, function(r) { proc(JSON.parse(r)) });
-        return;
+//        Jemplate.Ajax.get(data, function(r) { proc(Jemplate.JSON.parse(r)) });
+        var url = data;
+        Jemplate.Ajax.processGet(url, function(data) { proc(data) });
+        return null;
     }
 
     return proc(data);
@@ -147,6 +154,14 @@ proto.set_error = function(error, output) {
     return error;
 }
 
+proto.plugin = function(name, args) {
+    if (typeof name == 'undefined')
+        throw "Unknown plugin name ':" + name + "'";
+
+    // The Context object (this) is passed as the first argument to the plugin.
+    return new window[name](this, args);
+}
+
 proto.filter = function(text, name, args) {
     if (name == 'null')
         name = "null_filter";
@@ -156,6 +171,17 @@ proto.filter = function(text, name, args) {
         throw "Unknown filter name ':" + name + "'";
 }
 
+//------------------------------------------------------------------------------
+// Jemplate.Plugin class
+//------------------------------------------------------------------------------
+if (typeof Jemplate.Plugin == 'undefined') {
+    Jemplate.Plugin = function() { };
+}
+
+proto = Jemplate.Plugin.prototype;
+
+proto.plugins = {};
+
 //------------------------------------------------------------------------------
 // Jemplate.Filter class
 //------------------------------------------------------------------------------
@@ -226,7 +252,7 @@ proto.filters.uri = function(text) {
 
 proto.filters.indent = function(text, args) {
     var pad = args[0];
-    if (! text) return;
+    if (! text) return null;
     if (typeof pad == 'undefined')
         pad = 4;
 
@@ -244,7 +270,7 @@ proto.filters.indent = function(text, args) {
 
 proto.filters.truncate = function(text, args) {
     var len = args[0];
-    if (! text) return;
+    if (! text) return null;
     if (! len)
         len = 32;
     // This should probably be <=, but TT just uses <
@@ -255,7 +281,7 @@ proto.filters.truncate = function(text, args) {
 }
 
 proto.filters.repeat = function(text, iter) {
-    if (! text) return;
+    if (! text) return null;
     if (! iter || iter == 0)
         iter = 1;
     if (iter == 1) return text
@@ -268,7 +294,7 @@ proto.filters.repeat = function(text, iter) {
 }
 
 proto.filters.replace = function(text, args) {
-    if (! text) return;
+    if (! text) return null;
     var re_search = args[0];
     var text_replace = args[1];
     if (! re_search)
@@ -359,7 +385,7 @@ proto._dotop = function(root, item, args) {
         (typeof root[item] != 'undefined')) {
         var value = root[item];
         if (typeof value == 'function')
-            value = value();
+            value = value.apply(root);
         return value;
     }
 
@@ -554,6 +580,7 @@ proto.list_functions.splice = function(list, args) {
         return list.splice(args[0], args[1]);
     if (args.length == 3)
         return list.splice(args[0], args[1], args[2]);
+    return null;
 }
 
 proto.list_functions.push = function(list, args) {
@@ -629,20 +656,21 @@ proto.hash_functions.keys = function(hash) {
 proto.hash_functions.list = function(hash, args) {
     var what = '';
     if ( args )
-        var what = args[0];
+        what = args[0];
 
     var list = new Array();
+    var key;
     if (what == 'keys')
-        for ( var key in hash )
+        for ( key in hash )
             list.push(key);
     else if (what == 'values')
-        for ( var key in hash )
+        for ( key in hash )
             list.push(hash[key]);
     else if (what == 'each')
-        for ( var key in hash )
+        for ( key in hash )
             list.push(key, hash[key]);
     else
-        for ( var key in hash )
+        for ( key in hash )
             list.push({ 'key': key, 'value': hash[key] });
 
     return list;
@@ -681,7 +709,11 @@ proto.hash_functions.values = function(hash) {
     return list;
 }
 
-
+//  delete
+proto.hash_functions.remove = function(hash, args) {
+    return delete hash[args[0]];
+}
+proto.hash_functions['delete'] = proto.hash_functions.remove;
 
 //------------------------------------------------------------------------------
 // Jemplate.Iterator class
@@ -747,366 +779,80 @@ proto.get_next = function(should_init) {
     return [null, true];
 }
 
-//------------------------------------------------------------------------------
-// Debugging Support
-//------------------------------------------------------------------------------
+var stubExplanation = "stub that doesn't do anything. Try including the jQuery, YUI, or XHR option when building the runtime";
 
-function XXX(msg) {
-    if (! confirm(msg))
-        throw("terminated...");
-    return msg;
-}
+Jemplate.Ajax = {
 
-function JJJ(obj) {
-    return XXX(JSON.stringify(obj));
-}
+    get: function(url, callback) {
+        throw("This is a Jemplate.Ajax.get " + stubExplanation);
+    },
 
-//------------------------------------------------------------------------------
-// Ajax support
-//------------------------------------------------------------------------------
-if (! this.Ajax) Ajax = {};
+    processGet: function(url, callback) {
+        throw("This is a Jemplate.Ajax.processGet " + stubExplanation);
+    },
 
-Ajax.get = function(url, callback) {
-    var req = new XMLHttpRequest();
-    req.open('GET', url, Boolean(callback));
-    return Ajax._send(req, null, callback);
-}
-
-Ajax.post = function(url, data, callback) {
-    var req = new XMLHttpRequest();
-    req.open('POST', url, Boolean(callback));
-    req.setRequestHeader(
-        'Content-Type',
-        'application/x-www-form-urlencoded'
-    );
-    return Ajax._send(req, data, callback);
-}
-
-Ajax._send = function(req, data, callback) {
-    if (callback) {
-        req.onreadystatechange = function() {
-            if (req.readyState == 4) {
-                if(req.status == 200)
-                    callback(req.responseText);
-            }
-        };
+    post: function(url, callback) {
+        throw("This is a Jemplate.Ajax.post " + stubExplanation);
     }
-    req.send(data);
-    if (!callback) {
-        if (req.status != 200)
-            throw('Request for "' + url +
-                  '" failed with status: ' + req.status);
-        return req.responseText;
+
+};
+
+Jemplate.JSON = {
+
+    parse: function(decodeValue) {
+        throw("This is a Jemplate.JSON.parse " + stubExplanation);
+    },
+
+    stringify: function(encodeValue) {
+        throw("This is a Jemplate.JSON.stringify " + stubExplanation);
     }
-}
 
-//------------------------------------------------------------------------------
-// Cross-Browser XMLHttpRequest v1.1
-//------------------------------------------------------------------------------
-/*
-Emulate Gecko 'XMLHttpRequest()' functionality in IE and Opera. Opera requires
-the Sun Java Runtime Environment <http://www.java.com/>.
+};
 
-by Andrew Gregory
-http://www.scss.com.au/family/andrew/webdesign/xmlhttprequest/
+}());
 
-This work is licensed under the Creative Commons Attribution License. To view a
-copy of this license, visit http://creativecommons.org/licenses/by/1.0/ or send
-a letter to Creative Commons, 559 Nathan Abbott Way, Stanford, California 94305,
-USA.
-*/
+;;(function(){
 
-// IE support
-if (window.ActiveXObject && !window.XMLHttpRequest) {
-  window.XMLHttpRequest = function() {
-    return new ActiveXObject((navigator.userAgent.toLowerCase().indexOf('msie 5') != -1) ? 'Microsoft.XMLHTTP' : 'Msxml2.XMLHTTP');
-  };
-}
+Jemplate.Ajax = {
 
-// Opera support
-if (window.opera && !window.XMLHttpRequest) {
-  window.XMLHttpRequest = function() {
-    this.readyState = 0; // 0=uninitialized,1=loading,2=loaded,3=interactive,4=complete
-    this.status = 0; // HTTP status codes
-    this.statusText = '';
-    this._headers = [];
-    this._aborted = false;
-    this._async = true;
-    this.abort = function() {
-      this._aborted = true;
-    };
-    this.getAllResponseHeaders = function() {
-      return this.getAllResponseHeader('*');
-    };
-    this.getAllResponseHeader = function(header) {
-      var ret = '';
-      for (var i = 0; i < this._headers.length; i++) {
-        if (header == '*' || this._headers[i].h == header) {
-          ret += this._headers[i].h + ': ' + this._headers[i].v + '\n';
+    get: function(url, callback) {
+        if (typeof callback == "function") {
+            callback = { success: callback };
         }
-      }
-      return ret;
-    };
-    this.setRequestHeader = function(header, value) {
-      this._headers[this._headers.length] = {h:header, v:value};
-    };
-    this.open = function(method, url, async, user, password) {
-      this.method = method;
-      this.url = url;
-      this._async = true;
-      this._aborted = false;
-      if (arguments.length >= 3) {
-        this._async = async;
-      }
-      if (arguments.length > 3) {
-        // user/password support requires a custom Authenticator class
-        opera.postError('XMLHttpRequest.open() - user/password not supported');
-      }
-      this._headers = [];
-      this.readyState = 1;
-      if (this.onreadystatechange) {
-        this.onreadystatechange();
-      }
-    };
-    this.send = function(data) {
-      if (!navigator.javaEnabled()) {
-        alert("XMLHttpRequest.send() - Java must be installed and enabled.");
-        return;
-      }
-      if (this._async) {
-        setTimeout(this._sendasync, 0, this, data);
-        // this is not really asynchronous and won't execute until the current
-        // execution context ends
-      } else {
-        this._sendsync(data);
-      }
+        YAHOO.connect.asyncRequest("GET", url, callback);
+    },
+
+    processGet: function(url, processor) {
+        this.get(url, function(responseText){
+            process(YAHOO.lang.JSON.parse(responseText));
+        });
+    },
+
+    post: function(url, data, callback) {
+        if (typeof callback == "function") {
+            callback = { success: callback };
+        }
+        YAHOO.connect.asyncRequest("POST", url, callback, data);
     }
-    this._sendasync = function(req, data) {
-      if (!req._aborted) {
-        req._sendsync(data);
-      }
-    };
-    this._sendsync = function(data) {
-      this.readyState = 2;
-      if (this.onreadystatechange) {
-        this.onreadystatechange();
-      }
-      // open connection
-      var url = new java.net.URL(new java.net.URL(window.location.href), this.url);
-      var conn = url.openConnection();
-      for (var i = 0; i < this._headers.length; i++) {
-        conn.setRequestProperty(this._headers[i].h, this._headers[i].v);
-      }
-      this._headers = [];
-      if (this.method == 'POST') {
-        // POST data
-        conn.setDoOutput(true);
-        var wr = new java.io.OutputStreamWriter(conn.getOutputStream());
-        wr.write(data);
-        wr.flush();
-        wr.close();
-      }
-      // read response headers
-      // NOTE: the getHeaderField() methods always return nulls for me :(
-      var gotContentEncoding = false;
-      var gotContentLength = false;
-      var gotContentType = false;
-      var gotDate = false;
-      var gotExpiration = false;
-      var gotLastModified = false;
-      for (var i = 0; ; i++) {
-        var hdrName = conn.getHeaderFieldKey(i);
-        var hdrValue = conn.getHeaderField(i);
-        if (hdrName == null && hdrValue == null) {
-          break;
-        }
-        if (hdrName != null) {
-          this._headers[this._headers.length] = {h:hdrName, v:hdrValue};
-          switch (hdrName.toLowerCase()) {
-            case 'content-encoding': gotContentEncoding = true; break;
-            case 'content-length'  : gotContentLength   = true; break;
-            case 'content-type'    : gotContentType     = true; break;
-            case 'date'            : gotDate            = true; break;
-            case 'expires'         : gotExpiration      = true; break;
-            case 'last-modified'   : gotLastModified    = true; break;
-          }
-        }
-      }
-      // try to fill in any missing header information
-      var val;
-      val = conn.getContentEncoding();
-      if (val != null && !gotContentEncoding) this._headers[this._headers.length] = {h:'Content-encoding', v:val};
-      val = conn.getContentLength();
-      if (val != -1 && !gotContentLength) this._headers[this._headers.length] = {h:'Content-length', v:val};
-      val = conn.getContentType();
-      if (val != null && !gotContentType) this._headers[this._headers.length] = {h:'Content-type', v:val};
-      val = conn.getDate();
-      if (val != 0 && !gotDate) this._headers[this._headers.length] = {h:'Date', v:(new Date(val)).toUTCString()};
-      val = conn.getExpiration();
-      if (val != 0 && !gotExpiration) this._headers[this._headers.length] = {h:'Expires', v:(new Date(val)).toUTCString()};
-      val = conn.getLastModified();
-      if (val != 0 && !gotLastModified) this._headers[this._headers.length] = {h:'Last-modified', v:(new Date(val)).toUTCString()};
-      // read response data
-      var reqdata = '';
-      var stream = conn.getInputStream();
-      if (stream) {
-        var reader = new java.io.BufferedReader(new java.io.InputStreamReader(stream));
-        var line;
-        while ((line = reader.readLine()) != null) {
-          if (this.readyState == 2) {
-            this.readyState = 3;
-            if (this.onreadystatechange) {
-              this.onreadystatechange();
-            }
-          }
-          reqdata += line + '\n';
-        }
-        reader.close();
-        this.status = 200;
-        this.statusText = 'OK';
-        this.responseText = reqdata;
-        this.readyState = 4;
-        if (this.onreadystatechange) {
-          this.onreadystatechange();
-        }
-        if (this.onload) {
-          this.onload();
-        }
-      } else {
-        // error
-        this.status = 404;
-        this.statusText = 'Not Found';
-        this.responseText = '';
-        this.readyState = 4;
-        if (this.onreadystatechange) {
-          this.onreadystatechange();
-        }
-        if (this.onerror) {
-          this.onerror();
-        }
-      }
-    };
-  };
-}
-// ActiveXObject emulation
-if (!window.ActiveXObject && window.XMLHttpRequest) {
-  window.ActiveXObject = function(type) {
-    switch (type.toLowerCase()) {
-      case 'microsoft.xmlhttp':
-      case 'msxml2.xmlhttp':
-        return new XMLHttpRequest();
+
+};
+
+}());
+
+;;(function(){
+
+Jemplate.JSON = {
+
+    parse: function(encoded) {
+        return YAHOO.lang.JSON.parse(encoded);
+    },
+
+    stringify: function(decoded) {
+        return YAHOO.lang.JSON.stringify(decoded);
     }
-    return null;
-  };
-}
 
+};
 
-//------------------------------------------------------------------------------
-// JSON Support
-//------------------------------------------------------------------------------
+}());
 
-/*
-Copyright (c) 2005 JSON.org
-*/
-var JSON = function () {
-    var m = {
-            '\b': '\\b',
-            '\t': '\\t',
-            '\n': '\\n',
-            '\f': '\\f',
-            '\r': '\\r',
-            '"' : '\\"',
-            '\\': '\\\\'
-        },
-        s = {
-            'boolean': function (x) {
-                return String(x);
-            },
-            number: function (x) {
-                return isFinite(x) ? String(x) : 'null';
-            },
-            string: function (x) {
-                if (/["\\\x00-\x1f]/.test(x)) {
-                    x = x.replace(/([\x00-\x1f\\"])/g, function(a, b) {
-                        var c = m[b];
-                        if (c) {
-                            return c;
-                        }
-                        c = b.charCodeAt();
-                        return '\\u00' +
-                            Math.floor(c / 16).toString(16) +
-                            (c % 16).toString(16);
-                    });
-                }
-                return '"' + x + '"';
-            },
-            object: function (x) {
-                if (x) {
-                    var a = [], b, f, i, l, v;
-                    if (x instanceof Array) {
-                        a[0] = '[';
-                        l = x.length;
-                        for (i = 0; i < l; i += 1) {
-                            v = x[i];
-                            f = s[typeof v];
-                            if (f) {
-                                v = f(v);
-                                if (typeof v == 'string') {
-                                    if (b) {
-                                        a[a.length] = ',';
-                                    }
-                                    a[a.length] = v;
-                                    b = true;
-                                }
-                            }
-                        }
-                        a[a.length] = ']';
-                    } else if (x instanceof Object) {
-                        a[0] = '{';
-                        for (i in x) {
-                            v = x[i];
-                            f = s[typeof v];
-                            if (f) {
-                                v = f(v);
-                                if (typeof v == 'string') {
-                                    if (b) {
-                                        a[a.length] = ',';
-                                    }
-                                    a.push(s.string(i), ':', v);
-                                    b = true;
-                                }
-                            }
-                        }
-                        a[a.length] = '}';
-                    } else {
-                        return;
-                    }
-                    return a.join('');
-                }
-                return 'null';
-            }
-        };
-    return {
-        copyright: '(c)2005 JSON.org',
-        license: 'http://www.crockford.com/JSON/license.html',
-        stringify: function (v) {
-            var f = s[typeof v];
-            if (f) {
-                v = f(v);
-                if (typeof v == 'string') {
-                    return v;
-                }
-            }
-            return null;
-        },
-        parse: function (text) {
-            try {
-                return !(/[^,:{}\[\]0-9.\-+Eaeflnr-u \n\r\t]/.test(
-                        text.replace(/"(\\.|[^"\\])*"/g, ''))) &&
-                    eval('(' + text + ')');
-            } catch (e) {
-                return false;
-            }
-        }
-    };
-}();
+1
\ No newline at end of file
